package pfc;

import java.io.*;
import java.util.*;

import javax.faces.bean.ApplicationScoped;
import javax.faces.bean.ManagedBean;
import javax.faces.context.FacesContext;

import org.coode.owlapi.manchesterowlsyntax.*;
import org.semanticweb.owlapi.apibinding.*;
import org.semanticweb.owlapi.expression.ParserException;
import org.semanticweb.owlapi.expression.ShortFormEntityChecker;
import org.semanticweb.owlapi.model.*;
import com.clarkparsia.pellet.owlapiv3.*;

import org.semanticweb.owlapi.util.*;



/* Aquesta classe conté les operacions d'accés a la informació desada en l'ontologia */

public class DataManager {
	private OWLOntologyManager manager;
	private OWLOntology deviceOntology;
	private PelletReasoner reasoner;
	
	public DataManager() {
		manager = OWLManager.createOWLOntologyManager();
		
		InputStream is = FacesContext.getCurrentInstance().getExternalContext().getResourceAsStream("/WEB-INF/dispositiusmobils.owl");
		try {
			deviceOntology = manager.loadOntologyFromOntologyDocument(is);
		} catch (OWLOntologyCreationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		reasoner = PelletReasonerFactory.getInstance().createReasoner(deviceOntology);
		reasoner.precomputeInferences();
	}
	
	
	// Obté la llista de dispositius (la propietat model) i reb com a paràmetre una expressió manchester per a selecionar dispositius
	
	private Set<String> getDeviceNames(String classExpression) {
		HashSet<String> names = new HashSet<String>();
		
		OWLDataFactory dataFactory = manager.getOWLDataFactory();
		PrefixManager pmCommon = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/common.owl#");
		
		BidirectionalShortFormProvider sfp = new BidirectionalShortFormProviderAdapter(manager.getOntologies(), new SimpleShortFormProvider());
		ManchesterOWLSyntaxClassExpressionParser parser = 
				new ManchesterOWLSyntaxClassExpressionParser(dataFactory, new ShortFormEntityChecker(sfp));
		
		
		OWLClassExpression devicesClExpr = null;
		try {
			devicesClExpr = parser.parse(classExpression);
		} catch (ParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Iterator<OWLNamedIndividual> itInd = reasoner.getInstances(devicesClExpr, false).getFlattened().iterator();
		while (itInd.hasNext()) {
			Iterator<OWLLiteral> itValues = reasoner.getDataPropertyValues(itInd.next(), dataFactory.getOWLDataProperty(":model", pmCommon)).iterator();
			if (itValues.hasNext()) {
				names.add(itValues.next().getLiteral());
			}
		}
		
		return names;
	}
	
	
	public Set<String> getDeviceNames() {
		return getDeviceNames("Device");
	}
	
	public Set<String> getDeviceNamesWithAccelerometer() {
		return getDeviceNames("Device and (deviceHardware some ( DeviceHardware and sensor some (Sensor and name value \"accelerometer\"^^string)))");
	}
	
	public Set<String> getDeviceNamesWithLightSensor() {
		return getDeviceNames("Device and (deviceHardware some ( DeviceHardware and sensor some (Sensor and name value \"light\"^^string)))");
	}
	
	public Set<String> getDeviceNamesWithHTML5Support() {
		return getDeviceNames("Device and (deviceSoftware some ( DeviceSoftware and  defaultWebBrowser some (WebBrowser and  supportedFormats some (Format and name value \"HTML\"^^string and version value \"5\"^^string))))");
	}
	
	public Set<String> getDeviceNamesWithFlash(boolean flash) {
		if (flash) {
			return getDeviceNames("Device and (deviceHardware some ( DeviceHardware and  primaryCamera some (Camera and  cameraFlash value true)))");
		} else
			return getDeviceNames("Device and (deviceHardware some ( DeviceHardware and  primaryCamera some (Camera and  cameraFlash value false)))");
	}
	
	public Set<String> getDeviceNamesWithSatelliteLocation() {
		return getDeviceNames("Device and (supportedLocationProviders some (LocationProvider and (locationMethod some SatelliteLocMethod)))");
	}
	
	public Set<String> getDeviceNamesWithWifi() {
		return getDeviceNames("Device and networkSupport some (NetworkSupport and supportedNetworkModes some (NetworkMode and networkTechnology some (NetworkTechnology and (name value \"IEEE 802.11a\"^^string or name value \"IEEE 802.11b\"^^string or name value \"IEEE 802.11g\"^^string or name value \"IEEE 802.11n\"^^string ) )))");
	}
	
	public Set<String> getDeviceNamesWithOS(String os) {
		return getDeviceNames("Device and deviceSoftware some (DeviceSoftware and operatingSystem some (OperatingSystem and name value \""
					+ os + "\"^^string))");
	}
	
	public Set<String> getDeviceNamesWithOSNoAndroidWindows() {
		return getDeviceNames("Device and deviceSoftware some (DeviceSoftware and operatingSystem exactly 0 (OperatingSystem and (name value \"Android\"^^string) or (name value \"Windows Phone\"^^string ) ))");
	}
	
	public Set<String> getDeviceNamesWithDisplayGreaterThan(float size) {
		return getDeviceNames("Device and deviceHardware some (DeviceHardware and display some (Display and displaySize some decimal[>\"" +
					size + "\"^^decimal]) )");
	}
	
	public Device getDeviceByModel(String model) {
		OWLDataFactory dataFactory = manager.getOWLDataFactory();

		PrefixManager pmCommon = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/common.owl#");
		PrefixManager pmHW = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/hardware.owl#");
		PrefixManager pmDM = new DefaultPrefixManager("http://www.semanticweb.org/ontologies/2011/11/dispositiusmobils.owl#");
		PrefixManager pmSW = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/software.owl#");
		
		BidirectionalShortFormProvider sfp = new BidirectionalShortFormProviderAdapter(manager.getOntologies(), new SimpleShortFormProvider());
		ManchesterOWLSyntaxClassExpressionParser parser = 
				new ManchesterOWLSyntaxClassExpressionParser(dataFactory, new ShortFormEntityChecker(sfp));
		
		String manchesterClassExpr = "Device and model value \"" + model + "\"";
		
		OWLClassExpression deviceByModel = null;
		try {
			deviceByModel = parser.parse(manchesterClassExpr);
		} catch (ParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		Iterator<OWLNamedIndividual> itInd = reasoner.getInstances(deviceByModel, true).getFlattened().iterator();
		Device d = new Device();
		if ( itInd.hasNext() ) {
			OWLNamedIndividual owlDevice = itInd.next();
				
			// Obtenim la propietat model del dispositiu
			Iterator<OWLLiteral> itValues = reasoner.getDataPropertyValues(owlDevice, dataFactory.getOWLDataProperty(":model", pmCommon)).iterator();
			if (itValues.hasNext()) {
				d.setModel(itValues.next().getLiteral());
			}
			// Obtenim la propietat url del dispositiu
			itValues = reasoner.getDataPropertyValues(owlDevice, dataFactory.getOWLDataProperty(":url", pmDM)).iterator();
			if (itValues.hasNext()) {
				d.setUrl((itValues.next().getLiteral()));
			}
			
			// Obtenim la propietat vendor del dispositiu i la propietat name del vendor 
			itInd = reasoner.getObjectPropertyValues(owlDevice, dataFactory.getOWLObjectProperty(":vendor", pmCommon)).getFlattened().iterator();
			if (itInd.hasNext()) {
				itValues = reasoner.getDataPropertyValues(itInd.next(), dataFactory.getOWLDataProperty(":name", pmCommon)).iterator();
				if (itValues.hasNext()) {
					d.setVendor(itValues.next().getLiteral());
				}
			}
			
			// Obtenim la propietat deviceHardware
			itInd = reasoner.getObjectPropertyValues(owlDevice, dataFactory.getOWLObjectProperty(":deviceHardware", pmHW)).getFlattened().iterator();
			if (itInd.hasNext()) {
				OWLNamedIndividual owlIndDeviceHardware = itInd.next();
				
				// Obtenim la propietat cpu i les propietats model i cpuFreq
				itInd = reasoner.getObjectPropertyValues(owlIndDeviceHardware, dataFactory.getOWLObjectProperty(":cpu", pmHW)).getFlattened().iterator();
				if (itInd.hasNext()) {
					OWLNamedIndividual owlIndCPU = itInd.next();
					 
					itValues = reasoner.getDataPropertyValues(owlIndCPU, dataFactory.getOWLDataProperty(":model", pmCommon)).iterator();
					if (itValues.hasNext()) {
						d.setCpu(itValues.next().getLiteral());
					}
						
					itValues = reasoner.getDataPropertyValues(owlIndCPU, dataFactory.getOWLDataProperty(":frequency", pmDM)).iterator();
					if (itValues.hasNext()) {
						d.setCpuFreq(itValues.next().parseInteger()/1000000);
					}
				}
				
				//Obtenim els valors de la propietat memory
				itInd = reasoner.getObjectPropertyValues(owlIndDeviceHardware, dataFactory.getOWLObjectProperty(":memory", pmHW)).getFlattened().iterator();
				while (itInd.hasNext()) {
					OWLNamedIndividual owlIndMemory = itInd.next(); 
					// Obtenim la propietat size
					itValues = reasoner.getDataPropertyValues(owlIndMemory, dataFactory.getOWLDataProperty(":size", pmHW)).iterator();
					if (itValues.hasNext()) {
						long size = Long.parseLong(itValues.next().getLiteral());
						// Si és memòria de tipus MainMemory
						if ( reasoner.getTypes(owlIndMemory, false).getFlattened().contains(dataFactory.getOWLClass(":MainMemory", pmDM)) ) {
							d.setRamMemory(d.getRamMemory() + size/(1024*1024));
						} // si és memòria de tipus StorageMemory
						else if ( reasoner.getTypes(owlIndMemory, false).getFlattened().contains(dataFactory.getOWLClass(":StorageMemory", pmDM)) ) {
							d.setStorageMemory(d.getStorageMemory() + size/1000000);
						}
					}	
				}
				
				// Obtenim l'objecte Display de la propietat display 
				itInd = reasoner.getObjectPropertyValues(owlIndDeviceHardware, dataFactory.getOWLObjectProperty(":display", pmHW)).getFlattened().iterator();
				while (itInd.hasNext()) {
					OWLNamedIndividual owlIndDisplay = itInd.next();
					// Obtenim les propietats resolutionWidth, resolutionHeight i displaySize
					
					itValues = reasoner.getDataPropertyValues(owlIndDisplay, dataFactory.getOWLDataProperty(":displaySize", pmDM)).iterator();
					if (itValues.hasNext()) {
						d.setDisplaySize(itValues.next().parseFloat());
					}
			
					
					itValues = reasoner.getDataPropertyValues(owlIndDisplay, dataFactory.getOWLDataProperty(":resolutionWidth", pmCommon)).iterator();
					if (itValues.hasNext()) {
						d.setResolutionWidth(itValues.next().parseInteger());
					}
					
					itValues = reasoner.getDataPropertyValues(owlIndDisplay, dataFactory.getOWLDataProperty(":resolutionHeight", pmCommon)).iterator();
					if (itValues.hasNext()) {
						d.setResolutionHeight(itValues.next().parseInteger());
					}
				}	
			}
			// Obtenim la propietat deviceSoftware
			itInd = reasoner.getObjectPropertyValues(owlDevice, dataFactory.getOWLObjectProperty(":deviceSoftware", pmSW)).getFlattened().iterator();
			while (itInd.hasNext()) {			
				OWLNamedIndividual owlIndDeviceSoftware = itInd.next();
				
				// Obtenim la propietat operatingSystem (cpu) i les propietats name (model) i version (cpuFreq)
				itInd = reasoner.getObjectPropertyValues(owlIndDeviceSoftware, dataFactory.getOWLObjectProperty(":operatingSystem", pmSW)).getFlattened().iterator();
				if (itInd.hasNext()) {
					OWLNamedIndividual owlIndOS = itInd.next();
					
					itValues = reasoner.getDataPropertyValues(owlIndOS, dataFactory.getOWLDataProperty(":name", pmCommon)).iterator();
					if (itValues.hasNext()) {
						d.setOperatingSystem(itValues.next().getLiteral());
					}
						
					itValues = reasoner.getDataPropertyValues(owlIndOS, dataFactory.getOWLDataProperty(":version", pmCommon)).iterator();
					if (itValues.hasNext()) {
						d.setOperatingSystem(d.getOperatingSystem() + " " + itValues.next().getLiteral());
					}
				}
			
			}
		}
		return d;
	}
	
	

	public Set<File> getFilesSupportedByModel(String model) {
		HashSet<File> files = new HashSet<File>();
		
		BidirectionalShortFormProvider sfp = new BidirectionalShortFormProviderAdapter(manager.getOntologies(), new SimpleShortFormProvider());
		ManchesterOWLSyntaxClassExpressionParser parser = 
				new ManchesterOWLSyntaxClassExpressionParser(manager.getOWLDataFactory(),	new ShortFormEntityChecker(sfp));
		
		// Construïm la DL query per a trobar els fitxers suportats per un dispositiu determinat
		String manchesterExpression = "File and inverse supports some (Device and model value \"" + model + "\")";
		
		try {
			OWLDataFactory dataFactory = manager.getOWLDataFactory();
			PrefixManager pmDM = new DefaultPrefixManager("http://www.semanticweb.org/ontologies/2011/11/dispositiusmobils.owl#");
			PrefixManager pmSW = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/software.owl#");
			PrefixManager pmCommon = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/common.owl#");
			
			OWLClassExpression supportedFiles = parser.parse(manchesterExpression);
			Set<OWLNamedIndividual> owlFiles = reasoner.getInstances(supportedFiles, true).getFlattened();
			Iterator<OWLNamedIndividual> itFiles = owlFiles.iterator();
			while (itFiles.hasNext()) {
				OWLNamedIndividual owlIndFile = itFiles.next();
				File f = new File();;
				
				// Obtenim l'objecte de tipus FileFormat de la propietat fileFormat
				OWLObjectProperty owlFileFormat = dataFactory.getOWLObjectProperty(":fileFormat", pmDM);
				Set<OWLNamedIndividual> owlIndFileFormatSet = reasoner.getObjectPropertyValues(owlIndFile, owlFileFormat).getFlattened();
				Iterator<OWLNamedIndividual> itInd = owlIndFileFormatSet.iterator();
				if (itInd.hasNext()){
					OWLNamedIndividual owlIndFileFormat = itInd.next();
					// Afegim a la llista un fitxer del tipus adequat AudioFile, VideoFile o ExecutableFile
					
					if ( reasoner.getTypes(owlIndFileFormat, true).getFlattened().contains(dataFactory.getOWLClass(":AudioFormat", pmSW)) ) {
						f = new AudioFile();
						((AudioFile) f).setBitrate(getFileBitrate(owlIndFileFormat));
						//((AudioFile) f).setBitrateType(getFileBitrateType(owlIndFileFormat));
						((AudioFile) f).setMediaContainer(getMediaContainer(owlIndFileFormat));
						((AudioFile) f).setAudioCodec(getAudioCodec(owlIndFileFormat));
					} 
					else if ( reasoner.getTypes(owlIndFileFormat, true).getFlattened().contains(dataFactory.getOWLClass(":VideoFormat", pmSW)) ) {
						f = new VideoFile();
						((VideoFile) f).setResolutionWidth(getResolutionWidth(owlIndFileFormat));
						((VideoFile) f).setResolutionHeight(getResolutionHeight(owlIndFileFormat));
						((VideoFile) f).setBitrate(getFileBitrate(owlIndFileFormat));
						//((VideoFile) f).setBitrateType(getFileBitrateType(owlIndFileFormat));
						((VideoFile) f).setVideoCodec(getVideoCodec(owlIndFileFormat));
						((VideoFile) f).setAudioCodec(getAudioCodec(owlIndFileFormat));
						((VideoFile) f).setMediaContainer(getMediaContainer(owlIndFileFormat));
					}
					else if ( reasoner.getTypes(owlIndFileFormat, true).getFlattened().contains(dataFactory.getOWLClass(":ImageFormat", pmSW)) ) {
						f = new ImageFile();
						((ImageFile) f).setResolutionWidth(getResolutionWidth(owlIndFileFormat));
						((ImageFile) f).setResolutionHeight(getResolutionHeight(owlIndFileFormat));
						((ImageFile) f).setMediaContainer(getMediaContainer(owlIndFileFormat));
					}
					else if ( reasoner.getTypes(owlIndFileFormat, true).getFlattened().contains(dataFactory.getOWLClass(":Executable", pmDM)) ) {
						f = new ExecutableFile();
						((ExecutableFile) f).setPlatform(getPlatform(owlIndFileFormat));
					} else {
						f = new OtherFile();
						((OtherFile) f).setFormat(getFormat(owlIndFileFormat));
					}
				
				} 
				
				
				//Obtenim la URL
				OWLDataProperty owlUrl = dataFactory.getOWLDataProperty(":url", pmDM);
				Set<OWLLiteral> urlValues = reasoner.getDataPropertyValues(owlIndFile, owlUrl);
				Iterator<OWLLiteral> itValues = urlValues.iterator();
				if ( itValues.hasNext() ) {
					f.setUrl(itValues.next().getLiteral());
				}
				
				files.add(f);				
			
			}
		} catch (ParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return files;
	}
	
	private String getFormat(OWLNamedIndividual fileFormat) {
		String format = new String();
		
		OWLDataFactory dataFactory = manager.getOWLDataFactory();
		PrefixManager pmCommon = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/common.owl#");
		
		OWLDataProperty owlName = dataFactory.getOWLDataProperty(":name", pmCommon);
		Set<OWLLiteral> nameValues = reasoner.getDataPropertyValues(fileFormat, owlName);
		Iterator<OWLLiteral> itValues = nameValues.iterator();
		if ( itValues.hasNext() ) {
			format = itValues.next().getLiteral();
		}
		OWLDataProperty owlVersion = dataFactory.getOWLDataProperty(":version", pmCommon);
		Set<OWLLiteral> versionValues = reasoner.getDataPropertyValues(fileFormat, owlVersion);
		itValues = versionValues.iterator();
		if ( itValues.hasNext() ) {
			format = format + itValues.next().getLiteral();
		}
		
		return format;
	}
		
	private long getFileBitrate(OWLNamedIndividual fileFormat) {
		OWLDataFactory dataFactory = manager.getOWLDataFactory();
		PrefixManager pmDM = new DefaultPrefixManager("http://www.semanticweb.org/ontologies/2011/11/dispositiusmobils.owl#");
		
		OWLDataProperty owlBitrate = dataFactory.getOWLDataProperty(":bitrate", pmDM);
		Set<OWLLiteral> urlValues = reasoner.getDataPropertyValues(fileFormat, owlBitrate);
		Iterator<OWLLiteral> itValues = urlValues.iterator();
		if ( itValues.hasNext() ) {
			return Long.parseLong(itValues.next().getLiteral());
		} else return 0;
	}
	
	private String getFileBitrateType(OWLNamedIndividual fileFormat) {
		OWLDataFactory dataFactory = manager.getOWLDataFactory();
		PrefixManager pmDM = new DefaultPrefixManager("http://www.semanticweb.org/ontologies/2011/11/dispositiusmobils.owl#");
		
		OWLObjectProperty owlBitrateType = dataFactory.getOWLObjectProperty(":bitrateType", pmDM);
		Set<OWLNamedIndividual> owlIndBitrateType = reasoner.getObjectPropertyValues(fileFormat, owlBitrateType).getFlattened();
		Iterator <OWLNamedIndividual> itInd = owlIndBitrateType.iterator();
		if (itInd.hasNext()) {
			if ( itInd.next().equals(dataFactory.getOWLNamedIndividual(":VBR", pmDM))) {
				return "VBR";
			}
		}
		return "CBR";
	}
	
	private long getResolutionWidth(OWLNamedIndividual fileFormat) {
		OWLDataFactory dataFactory = manager.getOWLDataFactory();
		PrefixManager pmCommon = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/common.owl#");
		
		OWLDataProperty owlResolutionWidth = dataFactory.getOWLDataProperty(":resolutionWidth", pmCommon);
		Set<OWLLiteral> urlValues = reasoner.getDataPropertyValues(fileFormat, owlResolutionWidth);
		Iterator<OWLLiteral> itValues = urlValues.iterator();
		if ( itValues.hasNext() ) {
			return Long.parseLong(itValues.next().getLiteral());
		} else return 0;
	}
	
	private long getResolutionHeight(OWLNamedIndividual fileFormat) {
		OWLDataFactory dataFactory = manager.getOWLDataFactory();
		PrefixManager pmCommon = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/common.owl#");
		
		OWLDataProperty owlResolutionWidth = dataFactory.getOWLDataProperty(":resolutionHeight", pmCommon);
		Set<OWLLiteral> urlValues = reasoner.getDataPropertyValues(fileFormat, owlResolutionWidth);
		Iterator<OWLLiteral> itValues = urlValues.iterator();
		if ( itValues.hasNext() ) {
			return Long.parseLong(itValues.next().getLiteral());
		} else return 0;
	}
	
	private String getMediaContainer(OWLNamedIndividual fileFormat) {
		OWLDataFactory dataFactory = manager.getOWLDataFactory();
		PrefixManager pmDM = new DefaultPrefixManager("http://www.semanticweb.org/ontologies/2011/11/dispositiusmobils.owl#");
		PrefixManager pmCommon = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/common.owl#");
		
		OWLObjectProperty owlMediaContainer = dataFactory.getOWLObjectProperty(":mediaContainer", pmDM);
		Set<OWLNamedIndividual> owlIndMediaContainer = reasoner.getObjectPropertyValues(fileFormat, owlMediaContainer).getFlattened();
		Iterator <OWLNamedIndividual> itInd = owlIndMediaContainer.iterator();
		if (itInd.hasNext()) {
			OWLDataProperty owlProp = dataFactory.getOWLDataProperty(":name", pmCommon);
			Set<OWLLiteral> owlPropValues = reasoner.getDataPropertyValues(itInd.next(), owlProp);
			Iterator<OWLLiteral> itValues = owlPropValues.iterator();
			if (itValues.hasNext()) {
				return itValues.next().getLiteral();
			}	
		}
		return "";
	}
	
	private String getVideoCodec(OWLNamedIndividual fileFormat) {
		OWLDataFactory dataFactory = manager.getOWLDataFactory();
		PrefixManager pmDM = new DefaultPrefixManager("http://www.semanticweb.org/ontologies/2011/11/dispositiusmobils.owl#");
		PrefixManager pmCommon = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/common.owl#");
		
		OWLObjectProperty owlVideoCodec = dataFactory.getOWLObjectProperty(":videoCodec", pmDM);
		Set<OWLNamedIndividual> owlIndVideoCodec = reasoner.getObjectPropertyValues(fileFormat, owlVideoCodec).getFlattened();
		Iterator <OWLNamedIndividual> itInd = owlIndVideoCodec.iterator();
		if (itInd.hasNext()) {
			OWLDataProperty owlProp = dataFactory.getOWLDataProperty(":name", pmCommon);
			Set<OWLLiteral> owlPropValues = reasoner.getDataPropertyValues(itInd.next(), owlProp);
			Iterator<OWLLiteral> itValues = owlPropValues.iterator();
			if (itValues.hasNext()) {
				return itValues.next().getLiteral();
			}	
		}
		return "";
	}
	
	private String getAudioCodec(OWLNamedIndividual fileFormat) {
		OWLDataFactory dataFactory = manager.getOWLDataFactory();
		PrefixManager pmDM = new DefaultPrefixManager("http://www.semanticweb.org/ontologies/2011/11/dispositiusmobils.owl#");
		PrefixManager pmCommon = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/common.owl#");
		
		OWLObjectProperty owlAudioCodec = dataFactory.getOWLObjectProperty(":audioCodec", pmDM);
		Set<OWLNamedIndividual> owlIndAudioCodec = reasoner.getObjectPropertyValues(fileFormat, owlAudioCodec).getFlattened();
		Iterator <OWLNamedIndividual> itInd = owlIndAudioCodec.iterator();
		if (itInd.hasNext()) {
			OWLDataProperty owlProp = dataFactory.getOWLDataProperty(":name", pmCommon);
			Set<OWLLiteral> owlPropValues = reasoner.getDataPropertyValues(itInd.next(), owlProp);
			Iterator<OWLLiteral> itValues = owlPropValues.iterator();
			if (itValues.hasNext()) {
				return itValues.next().getLiteral();
			}	
		}
		return "";
	}
	
	private String getImageCodec(OWLNamedIndividual fileFormat) {
		OWLDataFactory dataFactory = manager.getOWLDataFactory();
		PrefixManager pmDM = new DefaultPrefixManager("http://www.semanticweb.org/ontologies/2011/11/dispositiusmobils.owl#");
		PrefixManager pmCommon = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/common.owl#");
		
		OWLObjectProperty owlImageCodec = dataFactory.getOWLObjectProperty(":imageCodec", pmDM);
		Set<OWLNamedIndividual> owlIndImageCodec = reasoner.getObjectPropertyValues(fileFormat, owlImageCodec).getFlattened();
		Iterator <OWLNamedIndividual> itInd = owlIndImageCodec.iterator();
		if (itInd.hasNext()) {
			OWLDataProperty owlProp = dataFactory.getOWLDataProperty(":name", pmCommon);
			Set<OWLLiteral> owlPropValues = reasoner.getDataPropertyValues(itInd.next(), owlProp);
			Iterator<OWLLiteral> itValues = owlPropValues.iterator();
			if (itValues.hasNext()) {
				return itValues.next().getLiteral();
			}	
		}
		return "";
	}
	
	private String getPlatform(OWLNamedIndividual fileFormat) {
		String osname = "", osversion = "";
		
		OWLDataFactory dataFactory = manager.getOWLDataFactory();
		PrefixManager pmCommon = new DefaultPrefixManager("http://www.w3.org/2007/uwa/context/common.owl#");
		PrefixManager pmDM = new DefaultPrefixManager("http://www.semanticweb.org/ontologies/2011/11/dispositiusmobils.owl#");
		
		OWLObjectProperty owlOperatingSystem = dataFactory.getOWLObjectProperty(":platform", pmDM);
		Set<OWLNamedIndividual> owlIndOperatingSystem = reasoner.getObjectPropertyValues(fileFormat, owlOperatingSystem).getFlattened();
		Iterator <OWLNamedIndividual> itInd = owlIndOperatingSystem.iterator();
		if (itInd.hasNext()) {
			OWLNamedIndividual os = itInd.next();
			
			OWLDataProperty owlProp = dataFactory.getOWLDataProperty(":name", pmCommon);
			Set<OWLLiteral> owlPropValues = reasoner.getDataPropertyValues(os, owlProp);
			Iterator<OWLLiteral> itValues = owlPropValues.iterator();
			if (itValues.hasNext()) {
				osname = itValues.next().getLiteral();
			}	
			
			owlProp = dataFactory.getOWLDataProperty(":version", pmCommon);
			owlPropValues = reasoner.getDataPropertyValues(os, owlProp);
			itValues = owlPropValues.iterator();
			if (itValues.hasNext()) {
				osversion = itValues.next().getLiteral();
			}
		}
		return osname + " " + osversion;
	}
	

}
